# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo
from .shared_params.new_maximum import NewMaximum
from .shared_params.new_minimum import NewMinimum
from .shared_params.new_usage_discount import NewUsageDiscount
from .shared_params.new_amount_discount import NewAmountDiscount
from .shared.billing_cycle_relative_date import BillingCycleRelativeDate
from .shared_params.new_allocation_price import NewAllocationPrice
from .shared_params.new_floating_bulk_price import NewFloatingBulkPrice
from .shared_params.new_floating_unit_price import NewFloatingUnitPrice
from .shared_params.new_percentage_discount import NewPercentageDiscount
from .shared_params.new_floating_matrix_price import NewFloatingMatrixPrice
from .shared_params.new_floating_tiered_price import NewFloatingTieredPrice
from .shared_params.new_floating_package_price import NewFloatingPackagePrice
from .shared_params.unit_conversion_rate_config import UnitConversionRateConfig
from .shared_params.tiered_conversion_rate_config import TieredConversionRateConfig
from .shared_params.new_billing_cycle_configuration import NewBillingCycleConfiguration
from .shared_params.new_floating_grouped_tiered_price import NewFloatingGroupedTieredPrice
from .shared_params.new_floating_tiered_package_price import NewFloatingTieredPackagePrice
from .shared_params.new_dimensional_price_configuration import NewDimensionalPriceConfiguration
from .shared_params.new_floating_minimum_composite_price import NewFloatingMinimumCompositePrice
from .shared_params.new_floating_unit_with_percent_price import NewFloatingUnitWithPercentPrice
from .shared_params.new_floating_grouped_allocation_price import NewFloatingGroupedAllocationPrice
from .shared_params.new_floating_bulk_with_proration_price import NewFloatingBulkWithProrationPrice
from .shared_params.new_floating_tiered_with_minimum_price import NewFloatingTieredWithMinimumPrice
from .shared_params.new_floating_unit_with_proration_price import NewFloatingUnitWithProrationPrice
from .shared_params.new_floating_tiered_with_proration_price import NewFloatingTieredWithProrationPrice
from .shared_params.new_floating_grouped_tiered_package_price import NewFloatingGroupedTieredPackagePrice
from .shared_params.new_floating_matrix_with_allocation_price import NewFloatingMatrixWithAllocationPrice
from .shared_params.new_floating_threshold_total_amount_price import NewFloatingThresholdTotalAmountPrice
from .shared_params.new_floating_cumulative_grouped_bulk_price import NewFloatingCumulativeGroupedBulkPrice
from .shared_params.new_floating_package_with_allocation_price import NewFloatingPackageWithAllocationPrice
from .shared_params.new_floating_matrix_with_display_name_price import NewFloatingMatrixWithDisplayNamePrice
from .shared_params.new_floating_max_group_tiered_package_price import NewFloatingMaxGroupTieredPackagePrice
from .shared_params.new_floating_tiered_package_with_minimum_price import NewFloatingTieredPackageWithMinimumPrice
from .shared_params.new_floating_grouped_with_metered_minimum_price import NewFloatingGroupedWithMeteredMinimumPrice
from .shared_params.new_floating_grouped_with_prorated_minimum_price import NewFloatingGroupedWithProratedMinimumPrice
from .shared_params.new_floating_scalable_matrix_with_unit_pricing_price import (
    NewFloatingScalableMatrixWithUnitPricingPrice,
)
from .shared_params.new_floating_scalable_matrix_with_tiered_pricing_price import (
    NewFloatingScalableMatrixWithTieredPricingPrice,
)

__all__ = [
    "SubscriptionPriceIntervalsParams",
    "Add",
    "AddDiscount",
    "AddDiscountAmountDiscountCreationParams",
    "AddDiscountPercentageDiscountCreationParams",
    "AddDiscountUsageDiscountCreationParams",
    "AddFixedFeeQuantityTransition",
    "AddPrice",
    "AddPriceNewFloatingGroupedWithMinMaxThresholdsPrice",
    "AddPriceNewFloatingGroupedWithMinMaxThresholdsPriceGroupedWithMinMaxThresholdsConfig",
    "AddPriceNewFloatingGroupedWithMinMaxThresholdsPriceConversionRateConfig",
    "AddAdjustment",
    "AddAdjustmentAdjustment",
    "Edit",
    "EditFixedFeeQuantityTransition",
    "EditAdjustment",
]


class SubscriptionPriceIntervalsParams(TypedDict, total=False):
    add: Iterable[Add]
    """A list of price intervals to add to the subscription."""

    add_adjustments: Iterable[AddAdjustment]
    """A list of adjustments to add to the subscription."""

    allow_invoice_credit_or_void: Optional[bool]
    """
    If false, this request will fail if it would void an issued invoice or create a
    credit note. Consider using this as a safety mechanism if you do not expect
    existing invoices to be changed.
    """

    edit: Iterable[Edit]
    """A list of price intervals to edit on the subscription."""

    edit_adjustments: Iterable[EditAdjustment]
    """A list of adjustments to edit on the subscription."""


class AddDiscountAmountDiscountCreationParams(TypedDict, total=False):
    amount_discount: Required[float]
    """Only available if discount_type is `amount`."""

    discount_type: Required[Literal["amount"]]


class AddDiscountPercentageDiscountCreationParams(TypedDict, total=False):
    discount_type: Required[Literal["percentage"]]

    percentage_discount: Required[float]
    """Only available if discount_type is `percentage`.

    This is a number between 0 and 1.
    """


class AddDiscountUsageDiscountCreationParams(TypedDict, total=False):
    discount_type: Required[Literal["usage"]]

    usage_discount: Required[float]
    """Only available if discount_type is `usage`.

    Number of usage units that this discount is for.
    """


AddDiscount: TypeAlias = Union[
    AddDiscountAmountDiscountCreationParams,
    AddDiscountPercentageDiscountCreationParams,
    AddDiscountUsageDiscountCreationParams,
]


class AddFixedFeeQuantityTransition(TypedDict, total=False):
    effective_date: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """The date that the fixed fee quantity transition should take effect."""

    quantity: Required[int]
    """The quantity of the fixed fee quantity transition."""


class AddPriceNewFloatingGroupedWithMinMaxThresholdsPriceGroupedWithMinMaxThresholdsConfig(TypedDict, total=False):
    grouping_key: Required[str]
    """The event property used to group before applying thresholds"""

    maximum_charge: Required[str]
    """The maximum amount to charge each group"""

    minimum_charge: Required[str]
    """The minimum amount to charge each group, regardless of usage"""

    per_unit_rate: Required[str]
    """The base price charged per group"""


AddPriceNewFloatingGroupedWithMinMaxThresholdsPriceConversionRateConfig: TypeAlias = Union[
    UnitConversionRateConfig, TieredConversionRateConfig
]


class AddPriceNewFloatingGroupedWithMinMaxThresholdsPrice(TypedDict, total=False):
    cadence: Required[Literal["annual", "semi_annual", "monthly", "quarterly", "one_time", "custom"]]
    """The cadence to bill for this price on."""

    currency: Required[str]
    """An ISO 4217 currency string for which this price is billed in."""

    grouped_with_min_max_thresholds_config: Required[
        AddPriceNewFloatingGroupedWithMinMaxThresholdsPriceGroupedWithMinMaxThresholdsConfig
    ]
    """Configuration for grouped_with_min_max_thresholds pricing"""

    item_id: Required[str]
    """The id of the item the price will be associated with."""

    model_type: Required[Literal["grouped_with_min_max_thresholds"]]
    """The pricing model type"""

    name: Required[str]
    """The name of the price."""

    billable_metric_id: Optional[str]
    """The id of the billable metric for the price.

    Only needed if the price is usage-based.
    """

    billed_in_advance: Optional[bool]
    """
    If the Price represents a fixed cost, the price will be billed in-advance if
    this is true, and in-arrears if this is false.
    """

    billing_cycle_configuration: Optional[NewBillingCycleConfiguration]
    """
    For custom cadence: specifies the duration of the billing period in days or
    months.
    """

    conversion_rate: Optional[float]
    """The per unit conversion rate of the price currency to the invoicing currency."""

    conversion_rate_config: Optional[AddPriceNewFloatingGroupedWithMinMaxThresholdsPriceConversionRateConfig]
    """The configuration for the rate of the price currency to the invoicing currency."""

    dimensional_price_configuration: Optional[NewDimensionalPriceConfiguration]
    """For dimensional price: specifies a price group and dimension values"""

    external_price_id: Optional[str]
    """An alias for the price."""

    fixed_price_quantity: Optional[float]
    """
    If the Price represents a fixed cost, this represents the quantity of units
    applied.
    """

    invoice_grouping_key: Optional[str]
    """The property used to group this price on an invoice"""

    invoicing_cycle_configuration: Optional[NewBillingCycleConfiguration]
    """Within each billing cycle, specifies the cadence at which invoices are produced.

    If unspecified, a single invoice is produced per billing cycle.
    """

    metadata: Optional[Dict[str, Optional[str]]]
    """User-specified key/value pairs for the resource.

    Individual keys can be removed by setting the value to `null`, and the entire
    metadata mapping can be cleared by setting `metadata` to `null`.
    """


AddPrice: TypeAlias = Union[
    NewFloatingUnitPrice,
    NewFloatingTieredPrice,
    NewFloatingBulkPrice,
    NewFloatingPackagePrice,
    NewFloatingMatrixPrice,
    NewFloatingThresholdTotalAmountPrice,
    NewFloatingTieredPackagePrice,
    NewFloatingTieredWithMinimumPrice,
    NewFloatingGroupedTieredPrice,
    NewFloatingTieredPackageWithMinimumPrice,
    NewFloatingPackageWithAllocationPrice,
    NewFloatingUnitWithPercentPrice,
    NewFloatingMatrixWithAllocationPrice,
    NewFloatingTieredWithProrationPrice,
    NewFloatingUnitWithProrationPrice,
    NewFloatingGroupedAllocationPrice,
    NewFloatingBulkWithProrationPrice,
    NewFloatingGroupedWithProratedMinimumPrice,
    NewFloatingGroupedWithMeteredMinimumPrice,
    AddPriceNewFloatingGroupedWithMinMaxThresholdsPrice,
    NewFloatingMatrixWithDisplayNamePrice,
    NewFloatingGroupedTieredPackagePrice,
    NewFloatingMaxGroupTieredPackagePrice,
    NewFloatingScalableMatrixWithUnitPricingPrice,
    NewFloatingScalableMatrixWithTieredPricingPrice,
    NewFloatingCumulativeGroupedBulkPrice,
    NewFloatingMinimumCompositePrice,
]


class Add(TypedDict, total=False):
    start_date: Required[
        Annotated[Union[Union[str, datetime], BillingCycleRelativeDate], PropertyInfo(format="iso8601")]
    ]
    """The start date of the price interval.

    This is the date that the price will start billing on the subscription.
    """

    allocation_price: Optional[NewAllocationPrice]
    """The definition of a new allocation price to create and add to the subscription."""

    discounts: Optional[Iterable[AddDiscount]]
    """A list of discounts to initialize on the price interval."""

    end_date: Annotated[Union[Union[str, datetime], BillingCycleRelativeDate, None], PropertyInfo(format="iso8601")]
    """The end date of the price interval.

    This is the date that the price will stop billing on the subscription.
    """

    external_price_id: Optional[str]
    """The external price id of the price to add to the subscription."""

    filter: Optional[str]
    """An additional filter to apply to usage queries.

    This filter must be expressed as a boolean
    [computed property](/extensibility/advanced-metrics#computed-properties). If
    null, usage queries will not include any additional filter.
    """

    fixed_fee_quantity_transitions: Optional[Iterable[AddFixedFeeQuantityTransition]]
    """A list of fixed fee quantity transitions to initialize on the price interval."""

    maximum_amount: Optional[float]
    """
    The maximum amount that will be billed for this price interval for a given
    billing period.
    """

    minimum_amount: Optional[float]
    """
    The minimum amount that will be billed for this price interval for a given
    billing period.
    """

    price: Optional[AddPrice]
    """New floating price request body params."""

    price_id: Optional[str]
    """The id of the price to add to the subscription."""

    usage_customer_ids: Optional[SequenceNotStr[str]]
    """
    A list of customer IDs whose usage events will be aggregated and billed under
    this subscription. By default, a subscription only considers usage events
    associated with its attached customer's customer_id. When usage_customer_ids is
    provided, the subscription includes usage events from the specified customers
    only. Provided usage_customer_ids must be either the customer for this
    subscription itself, or any of that customer's children.
    """


AddAdjustmentAdjustment: TypeAlias = Union[
    NewPercentageDiscount, NewUsageDiscount, NewAmountDiscount, NewMinimum, NewMaximum
]


class AddAdjustment(TypedDict, total=False):
    start_date: Required[
        Annotated[Union[Union[str, datetime], BillingCycleRelativeDate], PropertyInfo(format="iso8601")]
    ]
    """The start date of the adjustment interval.

    This is the date that the adjustment will start affecting prices on the
    subscription. The adjustment will apply to invoice dates that overlap with this
    `start_date`. This `start_date` is treated as inclusive for in-advance prices,
    and exclusive for in-arrears prices.
    """

    adjustment: Optional[AddAdjustmentAdjustment]
    """The definition of a new adjustment to create and add to the subscription."""

    adjustment_id: Optional[str]
    """The ID of the adjustment to add to the subscription.

    Adjustment IDs can be re-used from existing subscriptions or plans, but
    adjustments associated with coupon redemptions cannot be re-used.
    """

    end_date: Annotated[Union[Union[str, datetime], BillingCycleRelativeDate, None], PropertyInfo(format="iso8601")]
    """The end date of the adjustment interval.

    This is the date that the adjustment will stop affecting prices on the
    subscription. The adjustment will apply to invoice dates that overlap with this
    `end_date`.This `end_date` is treated as exclusive for in-advance prices, and
    inclusive for in-arrears prices.
    """


class EditFixedFeeQuantityTransition(TypedDict, total=False):
    effective_date: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]
    """The date that the fixed fee quantity transition should take effect."""

    quantity: Required[int]
    """The quantity of the fixed fee quantity transition."""


class Edit(TypedDict, total=False):
    price_interval_id: Required[str]
    """The id of the price interval to edit."""

    billing_cycle_day: Optional[int]
    """The updated billing cycle day for this price interval.

    If not specified, the billing cycle day will not be updated. Note that
    overlapping price intervals must have the same billing cycle day.
    """

    end_date: Annotated[Union[Union[str, datetime], BillingCycleRelativeDate, None], PropertyInfo(format="iso8601")]
    """The updated end date of this price interval.

    If not specified, the end date will not be updated.
    """

    filter: Optional[str]
    """An additional filter to apply to usage queries.

    This filter must be expressed as a boolean
    [computed property](/extensibility/advanced-metrics#computed-properties). If
    null, usage queries will not include any additional filter.
    """

    fixed_fee_quantity_transitions: Optional[Iterable[EditFixedFeeQuantityTransition]]
    """A list of fixed fee quantity transitions to use for this price interval.

    Note that this list will overwrite all existing fixed fee quantity transitions
    on the price interval.
    """

    start_date: Annotated[Union[Union[str, datetime], BillingCycleRelativeDate], PropertyInfo(format="iso8601")]
    """The updated start date of this price interval.

    If not specified, the start date will not be updated.
    """

    usage_customer_ids: Optional[SequenceNotStr[str]]
    """
    A list of customer IDs whose usage events will be aggregated and billed under
    this subscription. By default, a subscription only considers usage events
    associated with its attached customer's customer_id. When usage_customer_ids is
    provided, the subscription includes usage events from the specified customers
    only. Provided usage_customer_ids must be either the customer for this
    subscription itself, or any of that customer's children.
    """


class EditAdjustment(TypedDict, total=False):
    adjustment_interval_id: Required[str]
    """The id of the adjustment interval to edit."""

    end_date: Annotated[Union[Union[str, datetime], BillingCycleRelativeDate, None], PropertyInfo(format="iso8601")]
    """The updated end date of this adjustment interval.

    If not specified, the end date will not be updated.
    """

    start_date: Annotated[Union[Union[str, datetime], BillingCycleRelativeDate], PropertyInfo(format="iso8601")]
    """The updated start date of this adjustment interval.

    If not specified, the start date will not be updated.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel
from .bulk_tier import BulkTier

__all__ = ["BulkConfig"]


class BulkConfig(BaseModel):
    tiers: List[BulkTier]
    """Bulk tiers for rating based on total usage volume"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["TopUpInvoiceSettings"]


class TopUpInvoiceSettings(BaseModel):
    auto_collection: bool
    """
    Whether the credits purchase invoice should auto collect with the customer's
    saved payment method.
    """

    net_terms: int
    """
    The net terms determines the difference between the invoice date and the issue
    date for the invoice. If you intend the invoice to be due on issue, set this
    to 0.
    """

    memo: Optional[str] = None
    """An optional memo to display on the invoice."""

    require_successful_payment: Optional[bool] = None
    """
    When true, credit blocks created by this top-up will require that the
    corresponding invoice is paid before they are drawn down from. If any topup
    block is pending payment, further automatic top-ups will be paused until the
    invoice is paid or voided.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["CustomerTaxID"]


class CustomerTaxID(TypedDict, total=False):
    country: Required[
        Literal[
            "AD",
            "AE",
            "AL",
            "AM",
            "AO",
            "AR",
            "AT",
            "AU",
            "AW",
            "AZ",
            "BA",
            "BB",
            "BD",
            "BE",
            "BF",
            "BG",
            "BH",
            "BJ",
            "BO",
            "BR",
            "BS",
            "BY",
            "CA",
            "CD",
            "CH",
            "CL",
            "CM",
            "CN",
            "CO",
            "CR",
            "CV",
            "DE",
            "CY",
            "CZ",
            "DK",
            "DO",
            "EC",
            "EE",
            "EG",
            "ES",
            "ET",
            "EU",
            "FI",
            "FR",
            "GB",
            "GE",
            "GN",
            "GR",
            "HK",
            "HR",
            "HU",
            "ID",
            "IE",
            "IL",
            "IN",
            "IS",
            "IT",
            "JP",
            "KE",
            "KG",
            "KH",
            "KR",
            "KZ",
            "LA",
            "LI",
            "LT",
            "LU",
            "LV",
            "MA",
            "MD",
            "ME",
            "MK",
            "MR",
            "MT",
            "MX",
            "MY",
            "NG",
            "NL",
            "NO",
            "NP",
            "NZ",
            "OM",
            "PE",
            "PH",
            "PL",
            "PT",
            "RO",
            "RS",
            "RU",
            "SA",
            "SE",
            "SG",
            "SI",
            "SK",
            "SN",
            "SR",
            "SV",
            "TH",
            "TJ",
            "TR",
            "TW",
            "TZ",
            "UA",
            "UG",
            "US",
            "UY",
            "UZ",
            "VE",
            "VN",
            "ZA",
            "ZM",
            "ZW",
        ]
    ]

    type: Required[
        Literal[
            "ad_nrt",
            "ae_trn",
            "al_tin",
            "am_tin",
            "ao_tin",
            "ar_cuit",
            "eu_vat",
            "au_abn",
            "au_arn",
            "aw_tin",
            "az_tin",
            "ba_tin",
            "bb_tin",
            "bd_bin",
            "bf_ifu",
            "bg_uic",
            "bh_vat",
            "bj_ifu",
            "bo_tin",
            "br_cnpj",
            "br_cpf",
            "bs_tin",
            "by_tin",
            "ca_bn",
            "ca_gst_hst",
            "ca_pst_bc",
            "ca_pst_mb",
            "ca_pst_sk",
            "ca_qst",
            "cd_nif",
            "ch_uid",
            "ch_vat",
            "cl_tin",
            "cm_niu",
            "cn_tin",
            "co_nit",
            "cr_tin",
            "cv_nif",
            "de_stn",
            "do_rcn",
            "ec_ruc",
            "eg_tin",
            "es_cif",
            "et_tin",
            "eu_oss_vat",
            "gb_vat",
            "ge_vat",
            "gn_nif",
            "hk_br",
            "hr_oib",
            "hu_tin",
            "id_npwp",
            "il_vat",
            "in_gst",
            "is_vat",
            "jp_cn",
            "jp_rn",
            "jp_trn",
            "ke_pin",
            "kg_tin",
            "kh_tin",
            "kr_brn",
            "kz_bin",
            "la_tin",
            "li_uid",
            "li_vat",
            "ma_vat",
            "md_vat",
            "me_pib",
            "mk_vat",
            "mr_nif",
            "mx_rfc",
            "my_frp",
            "my_itn",
            "my_sst",
            "ng_tin",
            "no_vat",
            "no_voec",
            "np_pan",
            "nz_gst",
            "om_vat",
            "pe_ruc",
            "ph_tin",
            "ro_tin",
            "rs_pib",
            "ru_inn",
            "ru_kpp",
            "sa_vat",
            "sg_gst",
            "sg_uen",
            "si_tin",
            "sn_ninea",
            "sr_fin",
            "sv_nit",
            "th_vat",
            "tj_tin",
            "tr_tin",
            "tw_vat",
            "tz_vat",
            "ua_vat",
            "ug_tin",
            "us_ein",
            "uy_ruc",
            "uz_tin",
            "uz_vat",
            "ve_rif",
            "vn_tin",
            "za_vat",
            "zm_tin",
            "zw_tin",
        ]
    ]

    value: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from .bulk_tier import BulkTier

__all__ = ["BulkConfig"]


class BulkConfig(TypedDict, total=False):
    tiers: Required[Iterable[BulkTier]]
    """Bulk tiers for rating based on total usage volume"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["SubscriptionTrialInfo"]


class SubscriptionTrialInfo(BaseModel):
    end_date: Optional[datetime] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Literal, Required, TypedDict

from .threshold_param import ThresholdParam

__all__ = ["AlertCreateForSubscriptionParams"]


class AlertCreateForSubscriptionParams(TypedDict, total=False):
    thresholds: Required[Iterable[ThresholdParam]]
    """The thresholds that define the values at which the alert will be triggered."""

    type: Required[Literal["usage_exceeded", "cost_exceeded"]]
    """The type of alert to create. This must be a valid alert type."""

    metric_id: Optional[str]
    """The metric to track usage for."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["AlertEnableParams"]


class AlertEnableParams(TypedDict, total=False):
    subscription_id: Optional[str]
    """Used to update the status of a plan alert scoped to this subscription_id"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel
from .transform_price_filter import TransformPriceFilter

__all__ = ["Maximum"]


class Maximum(BaseModel):
    applies_to_price_ids: List[str]
    """List of price_ids that this maximum amount applies to.

    For plan/plan phase maximums, this can be a subset of prices.
    """

    filters: List[TransformPriceFilter]
    """The filters that determine which prices to apply this maximum to."""

    maximum_amount: str
    """Maximum amount applied"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .unit_conversion_rate_config import UnitConversionRateConfig
from .tiered_conversion_rate_config import TieredConversionRateConfig
from .new_billing_cycle_configuration import NewBillingCycleConfiguration
from .new_dimensional_price_configuration import NewDimensionalPriceConfiguration

__all__ = [
    "NewPlanMaxGroupTieredPackagePrice",
    "MaxGroupTieredPackageConfig",
    "MaxGroupTieredPackageConfigTier",
    "ConversionRateConfig",
]


class MaxGroupTieredPackageConfigTier(TypedDict, total=False):
    tier_lower_bound: Required[str]
    """Tier lower bound"""

    unit_amount: Required[str]
    """Per unit amount"""


class MaxGroupTieredPackageConfig(TypedDict, total=False):
    grouping_key: Required[str]
    """
    The event property used to group before tiering the group with the highest value
    """

    package_size: Required[str]
    """Package size"""

    tiers: Required[Iterable[MaxGroupTieredPackageConfigTier]]
    """Apply tiered pricing to the largest group after grouping with the provided key."""


ConversionRateConfig: TypeAlias = Union[UnitConversionRateConfig, TieredConversionRateConfig]


class NewPlanMaxGroupTieredPackagePrice(TypedDict, total=False):
    cadence: Required[Literal["annual", "semi_annual", "monthly", "quarterly", "one_time", "custom"]]
    """The cadence to bill for this price on."""

    item_id: Required[str]
    """The id of the item the price will be associated with."""

    max_group_tiered_package_config: Required[MaxGroupTieredPackageConfig]
    """Configuration for max_group_tiered_package pricing"""

    model_type: Required[Literal["max_group_tiered_package"]]
    """The pricing model type"""

    name: Required[str]
    """The name of the price."""

    billable_metric_id: Optional[str]
    """The id of the billable metric for the price.

    Only needed if the price is usage-based.
    """

    billed_in_advance: Optional[bool]
    """
    If the Price represents a fixed cost, the price will be billed in-advance if
    this is true, and in-arrears if this is false.
    """

    billing_cycle_configuration: Optional[NewBillingCycleConfiguration]
    """
    For custom cadence: specifies the duration of the billing period in days or
    months.
    """

    conversion_rate: Optional[float]
    """The per unit conversion rate of the price currency to the invoicing currency."""

    conversion_rate_config: Optional[ConversionRateConfig]
    """The configuration for the rate of the price currency to the invoicing currency."""

    currency: Optional[str]
    """
    An ISO 4217 currency string, or custom pricing unit identifier, in which this
    price is billed.
    """

    dimensional_price_configuration: Optional[NewDimensionalPriceConfiguration]
    """For dimensional price: specifies a price group and dimension values"""

    external_price_id: Optional[str]
    """An alias for the price."""

    fixed_price_quantity: Optional[float]
    """
    If the Price represents a fixed cost, this represents the quantity of units
    applied.
    """

    invoice_grouping_key: Optional[str]
    """The property used to group this price on an invoice"""

    invoicing_cycle_configuration: Optional[NewBillingCycleConfiguration]
    """Within each billing cycle, specifies the cadence at which invoices are produced.

    If unspecified, a single invoice is produced per billing cycle.
    """

    metadata: Optional[Dict[str, Optional[str]]]
    """User-specified key/value pairs for the resource.

    Individual keys can be removed by setting the value to `null`, and the entire
    metadata mapping can be cleared by setting `metadata` to `null`.
    """

    reference_id: Optional[str]
    """
    A transient ID that can be used to reference this price when adding adjustments
    in the same API call.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr
from .transform_price_filter import TransformPriceFilter

__all__ = ["NewAmountDiscount"]


class NewAmountDiscount(TypedDict, total=False):
    adjustment_type: Required[Literal["amount_discount"]]

    amount_discount: Required[str]

    applies_to_all: Optional[Literal[True]]
    """If set, the adjustment will apply to every price on the subscription."""

    applies_to_item_ids: Optional[SequenceNotStr[str]]
    """The set of item IDs to which this adjustment applies."""

    applies_to_price_ids: Optional[SequenceNotStr[str]]
    """The set of price IDs to which this adjustment applies."""

    currency: Optional[str]
    """If set, only prices in the specified currency will have the adjustment applied."""

    filters: Optional[Iterable[TransformPriceFilter]]
    """A list of filters that determine which prices this adjustment will apply to."""

    is_invoice_level: bool
    """When false, this adjustment will be applied to a single price.

    Otherwise, it will be applied at the invoice level, possibly to multiple prices.
    """

    price_type: Optional[Literal["usage", "fixed_in_advance", "fixed_in_arrears", "fixed", "in_arrears"]]
    """If set, only prices of the specified type will have the adjustment applied."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["SubLineItemGrouping"]


class SubLineItemGrouping(BaseModel):
    key: str

    value: Optional[str] = None
    """No value indicates the default group"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from .plan import Plan
from .._utils import PropertyInfo
from .._models import BaseModel
from .customer import Customer
from .shared.price_interval import PriceInterval
from .shared.maximum_interval import MaximumInterval
from .shared.minimum_interval import MinimumInterval
from .shared.coupon_redemption import CouponRedemption
from .shared.adjustment_interval import AdjustmentInterval
from .shared.subscription_trial_info import SubscriptionTrialInfo
from .shared.usage_discount_interval import UsageDiscountInterval
from .shared.amount_discount_interval import AmountDiscountInterval
from .shared.percentage_discount_interval import PercentageDiscountInterval
from .shared.subscription_change_minified import SubscriptionChangeMinified
from .shared.fixed_fee_quantity_schedule_entry import FixedFeeQuantityScheduleEntry
from .shared.billing_cycle_anchor_configuration import BillingCycleAnchorConfiguration

__all__ = ["Subscription", "DiscountInterval"]

DiscountInterval: TypeAlias = Annotated[
    Union[AmountDiscountInterval, PercentageDiscountInterval, UsageDiscountInterval],
    PropertyInfo(discriminator="discount_type"),
]


class Subscription(BaseModel):
    id: str

    active_plan_phase_order: Optional[int] = None
    """
    The current plan phase that is active, only if the subscription's plan has
    phases.
    """

    adjustment_intervals: List[AdjustmentInterval]
    """
    The adjustment intervals for this subscription sorted by the start_date of the
    adjustment interval.
    """

    auto_collection: Optional[bool] = None
    """
    Determines whether issued invoices for this subscription will automatically be
    charged with the saved payment method on the due date. This property defaults to
    the plan's behavior. If null, defaults to the customer's setting.
    """

    billing_cycle_anchor_configuration: BillingCycleAnchorConfiguration

    billing_cycle_day: int
    """The day of the month on which the billing cycle is anchored.

    If the maximum number of days in a month is greater than this value, the last
    day of the month is the billing cycle day (e.g. billing_cycle_day=31 for April
    means the billing period begins on the 30th.
    """

    created_at: datetime

    current_billing_period_end_date: Optional[datetime] = None
    """The end of the current billing period.

    This is an exclusive timestamp, such that the instant returned is not part of
    the billing period. Set to null for subscriptions that are not currently active.
    """

    current_billing_period_start_date: Optional[datetime] = None
    """The start date of the current billing period.

    This is an inclusive timestamp; the instant returned is exactly the beginning of
    the billing period. Set to null if the subscription is not currently active.
    """

    customer: Customer
    """
    A customer is a buyer of your products, and the other party to the billing
    relationship.

    In Orb, customers are assigned system generated identifiers automatically, but
    it's often desirable to have these match existing identifiers in your system. To
    avoid having to denormalize Orb ID information, you can pass in an
    `external_customer_id` with your own identifier. See
    [Customer ID Aliases](/events-and-metrics/customer-aliases) for further
    information about how these aliases work in Orb.

    In addition to having an identifier in your system, a customer may exist in a
    payment provider solution like Stripe. Use the `payment_provider_id` and the
    `payment_provider` enum field to express this mapping.

    A customer also has a timezone (from the standard
    [IANA timezone database](https://www.iana.org/time-zones)), which defaults to
    your account's timezone. See [Timezone localization](/essentials/timezones) for
    information on what this timezone parameter influences within Orb.
    """

    default_invoice_memo: Optional[str] = None
    """Determines the default memo on this subscriptions' invoices.

    Note that if this is not provided, it is determined by the plan configuration.
    """

    discount_intervals: List[DiscountInterval]
    """The discount intervals for this subscription sorted by the start_date.

    This field is deprecated in favor of `adjustment_intervals`.
    """

    end_date: Optional[datetime] = None
    """The date Orb stops billing for this subscription."""

    fixed_fee_quantity_schedule: List[FixedFeeQuantityScheduleEntry]

    invoicing_threshold: Optional[str] = None

    maximum_intervals: List[MaximumInterval]
    """The maximum intervals for this subscription sorted by the start_date.

    This field is deprecated in favor of `adjustment_intervals`.
    """

    metadata: Dict[str, str]
    """User specified key-value pairs for the resource.

    If not present, this defaults to an empty dictionary. Individual keys can be
    removed by setting the value to `null`, and the entire metadata mapping can be
    cleared by setting `metadata` to `null`.
    """

    minimum_intervals: List[MinimumInterval]
    """The minimum intervals for this subscription sorted by the start_date.

    This field is deprecated in favor of `adjustment_intervals`.
    """

    name: str
    """The name of the subscription."""

    net_terms: int
    """
    Determines the difference between the invoice issue date for subscription
    invoices as the date that they are due. A value of `0` here represents that the
    invoice is due on issue, whereas a value of `30` represents that the customer
    has a month to pay the invoice.
    """

    pending_subscription_change: Optional[SubscriptionChangeMinified] = None
    """A pending subscription change if one exists on this subscription."""

    plan: Optional[Plan] = None
    """
    The [Plan](/core-concepts#plan-and-price) resource represents a plan that can be
    subscribed to by a customer. Plans define the billing behavior of the
    subscription. You can see more about how to configure prices in the
    [Price resource](/reference/price).
    """

    price_intervals: List[PriceInterval]
    """The price intervals for this subscription."""

    redeemed_coupon: Optional[CouponRedemption] = None

    start_date: datetime
    """The date Orb starts billing for this subscription."""

    status: Literal["active", "ended", "upcoming"]

    trial_info: SubscriptionTrialInfo



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["MatrixValue"]


class MatrixValue(BaseModel):
    dimension_values: List[Optional[str]]
    """One or two matrix keys to filter usage to this Matrix value by"""

    unit_amount: str
    """Unit price for the specified dimension_values"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from datetime import date
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["CreditNoteCreateParams", "LineItem"]


class CreditNoteCreateParams(TypedDict, total=False):
    line_items: Required[Iterable[LineItem]]

    reason: Required[Literal["duplicate", "fraudulent", "order_change", "product_unsatisfactory"]]
    """An optional reason for the credit note."""

    end_date: Annotated[Union[str, date, None], PropertyInfo(format="iso8601")]
    """
    A date string to specify the global credit note service period end date in the
    customer's timezone. This will be applied to all line items that don't have
    their own individual service periods specified. If not provided, line items will
    use their original invoice line item service periods. This date is inclusive.
    """

    memo: Optional[str]
    """An optional memo to attach to the credit note."""

    start_date: Annotated[Union[str, date, None], PropertyInfo(format="iso8601")]
    """
    A date string to specify the global credit note service period start date in the
    customer's timezone. This will be applied to all line items that don't have
    their own individual service periods specified. If not provided, line items will
    use their original invoice line item service periods. This date is inclusive.
    """


class LineItem(TypedDict, total=False):
    amount: Required[str]
    """The total amount in the invoice's currency to credit this line item."""

    invoice_line_item_id: Required[str]
    """The ID of the line item to credit."""

    end_date: Annotated[Union[str, date, None], PropertyInfo(format="iso8601")]
    """
    A date string to specify this line item's credit note service period end date in
    the customer's timezone. If provided, this will be used for this specific line
    item. If not provided, will use the global end_date if available, otherwise
    defaults to the original invoice line item's end date. This date is inclusive.
    """

    start_date: Annotated[Union[str, date, None], PropertyInfo(format="iso8601")]
    """
    A date string to specify this line item's credit note service period start date
    in the customer's timezone. If provided, this will be used for this specific
    line item. If not provided, will use the global start_date if available,
    otherwise defaults to the original invoice line item's start date. This date is
    inclusive.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from orb import Orb, AsyncOrb
from orb.types import Plan, PlanVersion
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestBeta:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create_plan_version(self, client: Orb) -> None:
        beta = client.beta.create_plan_version(
            plan_id="plan_id",
            version=0,
        )
        assert_matches_type(PlanVersion, beta, path=["response"])

    @parametrize
    def test_method_create_plan_version_with_all_params(self, client: Orb) -> None:
        beta = client.beta.create_plan_version(
            plan_id="plan_id",
            version=0,
            add_adjustments=[
                {
                    "adjustment": {
                        "adjustment_type": "percentage_discount",
                        "percentage_discount": 0,
                        "applies_to_all": True,
                        "applies_to_item_ids": ["item_1", "item_2"],
                        "applies_to_price_ids": ["price_1", "price_2"],
                        "currency": "currency",
                        "filters": [
                            {
                                "field": "price_id",
                                "operator": "includes",
                                "values": ["string"],
                            }
                        ],
                        "is_invoice_level": True,
                        "price_type": "usage",
                    },
                    "plan_phase_order": 0,
                }
            ],
            add_prices=[
                {
                    "allocation_price": {
                        "amount": "10.00",
                        "cadence": "monthly",
                        "currency": "USD",
                        "custom_expiration": {
                            "duration": 0,
                            "duration_unit": "day",
                        },
                        "expires_at_end_of_cadence": True,
                    },
                    "plan_phase_order": 0,
                    "price": {
                        "cadence": "annual",
                        "item_id": "item_id",
                        "model_type": "unit",
                        "name": "Annual fee",
                        "unit_config": {
                            "unit_amount": "unit_amount",
                            "scaling_factor": 0,
                        },
                        "billable_metric_id": "billable_metric_id",
                        "billed_in_advance": True,
                        "billing_cycle_configuration": {
                            "duration": 0,
                            "duration_unit": "day",
                        },
                        "conversion_rate": 0,
                        "conversion_rate_config": {
                            "conversion_rate_type": "unit",
                            "unit_config": {"unit_amount": "unit_amount"},
                        },
                        "currency": "currency",
                        "dimensional_price_configuration": {
                            "dimension_values": ["string"],
                            "dimensional_price_group_id": "dimensional_price_group_id",
                            "external_dimensional_price_group_id": "external_dimensional_price_group_id",
                        },
                        "external_price_id": "external_price_id",
                        "fixed_price_quantity": 0,
                        "invoice_grouping_key": "x",
                        "invoicing_cycle_configuration": {
                            "duration": 0,
                            "duration_unit": "day",
                        },
                        "metadata": {"foo": "string"},
                        "reference_id": "reference_id",
                    },
                }
            ],
            remove_adjustments=[
                {
                    "adjustment_id": "adjustment_id",
                    "plan_phase_order": 0,
                }
            ],
            remove_prices=[
                {
                    "price_id": "price_id",
                    "plan_phase_order": 0,
                }
            ],
            replace_adjustments=[
                {
                    "adjustment": {
                        "adjustment_type": "percentage_discount",
                        "percentage_discount": 0,
                        "applies_to_all": True,
                        "applies_to_item_ids": ["item_1", "item_2"],
                        "applies_to_price_ids": ["price_1", "price_2"],
                        "currency": "currency",
                        "filters": [
                            {
                                "field": "price_id",
                                "operator": "includes",
                                "values": ["string"],
                            }
                        ],
                        "is_invoice_level": True,
                        "price_type": "usage",
                    },
                    "replaces_adjustment_id": "replaces_adjustment_id",
                    "plan_phase_order": 0,
                }
            ],
            replace_prices=[
                {
                    "replaces_price_id": "replaces_price_id",
                    "allocation_price": {
                        "amount": "10.00",
                        "cadence": "monthly",
                        "currency": "USD",
                        "custom_expiration": {
                            "duration": 0,
                            "duration_unit": "day",
                        },
                        "expires_at_end_of_cadence": True,
                    },
                    "plan_phase_order": 0,
                    "price": {
                        "cadence": "annual",
                        "item_id": "item_id",
                        "model_type": "unit",
                        "name": "Annual fee",
                        "unit_config": {
                            "unit_amount": "unit_amount",
                            "scaling_factor": 0,
                        },
                        "billable_metric_id": "billable_metric_id",
                        "billed_in_advance": True,
                        "billing_cycle_configuration": {
                            "duration": 0,
                            "duration_unit": "day",
                        },
                        "conversion_rate": 0,
                        "conversion_rate_config": {
                            "conversion_rate_type": "unit",
                            "unit_config": {"unit_amount": "unit_amount"},
                        },
                        "currency": "currency",
                        "dimensional_price_configuration": {
                            "dimension_values": ["string"],
                            "dimensional_price_group_id": "dimensional_price_group_id",
                            "external_dimensional_price_group_id": "external_dimensional_price_group_id",
                        },
                        "external_price_id": "external_price_id",
                        "fixed_price_quantity": 0,
                        "invoice_grouping_key": "x",
                        "invoicing_cycle_configuration": {
                            "duration": 0,
                            "duration_unit": "day",
                        },
                        "metadata": {"foo": "string"},
                        "reference_id": "reference_id",
                    },
                }
            ],
            set_as_default=True,
        )
        assert_matches_type(PlanVersion, beta, path=["response"])

    @parametrize
    def test_raw_response_create_plan_version(self, client: Orb) -> None:
        response = client.beta.with_raw_response.create_plan_version(
            plan_id="plan_id",
            version=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        beta = response.parse()
        assert_matches_type(PlanVersion, beta, path=["response"])

    @parametrize
    def test_streaming_response_create_plan_version(self, client: Orb) -> None:
        with client.beta.with_streaming_response.create_plan_version(
            plan_id="plan_id",
            version=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            beta = response.parse()
            assert_matches_type(PlanVersion, beta, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create_plan_version(self, client: Orb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `plan_id` but received ''"):
            client.beta.with_raw_response.create_plan_version(
                plan_id="",
                version=0,
            )

    @parametrize
    def test_method_fetch_plan_version(self, client: Orb) -> None:
        beta = client.beta.fetch_plan_version(
            version="version",
            plan_id="plan_id",
        )
        assert_matches_type(PlanVersion, beta, path=["response"])

    @parametrize
    def test_raw_response_fetch_plan_version(self, client: Orb) -> None:
        response = client.beta.with_raw_response.fetch_plan_version(
            version="version",
            plan_id="plan_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        beta = response.parse()
        assert_matches_type(PlanVersion, beta, path=["response"])

    @parametrize
    def test_streaming_response_fetch_plan_version(self, client: Orb) -> None:
        with client.beta.with_streaming_response.fetch_plan_version(
            version="version",
            plan_id="plan_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            beta = response.parse()
            assert_matches_type(PlanVersion, beta, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_fetch_plan_version(self, client: Orb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `plan_id` but received ''"):
            client.beta.with_raw_response.fetch_plan_version(
                version="version",
                plan_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `version` but received ''"):
            client.beta.with_raw_response.fetch_plan_version(
                version="",
                plan_id="plan_id",
            )

    @parametrize
    def test_method_set_default_plan_version(self, client: Orb) -> None:
        beta = client.beta.set_default_plan_version(
            plan_id="plan_id",
            version=0,
        )
        assert_matches_type(Plan, beta, path=["response"])

    @parametrize
    def test_raw_response_set_default_plan_version(self, client: Orb) -> None:
        response = client.beta.with_raw_response.set_default_plan_version(
            plan_id="plan_id",
            version=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        beta = response.parse()
        assert_matches_type(Plan, beta, path=["response"])

    @parametrize
    def test_streaming_response_set_default_plan_version(self, client: Orb) -> None:
        with client.beta.with_streaming_response.set_default_plan_version(
            plan_id="plan_id",
            version=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            beta = response.parse()
            assert_matches_type(Plan, beta, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_set_default_plan_version(self, client: Orb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `plan_id` but received ''"):
            client.beta.with_raw_response.set_default_plan_version(
                plan_id="",
                version=0,
            )


class TestAsyncBeta:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create_plan_version(self, async_client: AsyncOrb) -> None:
        beta = await async_client.beta.create_plan_version(
            plan_id="plan_id",
            version=0,
        )
        assert_matches_type(PlanVersion, beta, path=["response"])

    @parametrize
    async def test_method_create_plan_version_with_all_params(self, async_client: AsyncOrb) -> None:
        beta = await async_client.beta.create_plan_version(
            plan_id="plan_id",
            version=0,
            add_adjustments=[
                {
                    "adjustment": {
                        "adjustment_type": "percentage_discount",
                        "percentage_discount": 0,
                        "applies_to_all": True,
                        "applies_to_item_ids": ["item_1", "item_2"],
                        "applies_to_price_ids": ["price_1", "price_2"],
                        "currency": "currency",
                        "filters": [
                            {
                                "field": "price_id",
                                "operator": "includes",
                                "values": ["string"],
                            }
                        ],
                        "is_invoice_level": True,
                        "price_type": "usage",
                    },
                    "plan_phase_order": 0,
                }
            ],
            add_prices=[
                {
                    "allocation_price": {
                        "amount": "10.00",
                        "cadence": "monthly",
                        "currency": "USD",
                        "custom_expiration": {
                            "duration": 0,
                            "duration_unit": "day",
                        },
                        "expires_at_end_of_cadence": True,
                    },
                    "plan_phase_order": 0,
                    "price": {
                        "cadence": "annual",
                        "item_id": "item_id",
                        "model_type": "unit",
                        "name": "Annual fee",
                        "unit_config": {
                            "unit_amount": "unit_amount",
                            "scaling_factor": 0,
                        },
                        "billable_metric_id": "billable_metric_id",
                        "billed_in_advance": True,
                        "billing_cycle_configuration": {
                            "duration": 0,
                            "duration_unit": "day",
                        },
                        "conversion_rate": 0,
                        "conversion_rate_config": {
                            "conversion_rate_type": "unit",
                            "unit_config": {"unit_amount": "unit_amount"},
                        },
                        "currency": "currency",
                        "dimensional_price_configuration": {
                            "dimension_values": ["string"],
                            "dimensional_price_group_id": "dimensional_price_group_id",
                            "external_dimensional_price_group_id": "external_dimensional_price_group_id",
                        },
                        "external_price_id": "external_price_id",
                        "fixed_price_quantity": 0,
                        "invoice_grouping_key": "x",
                        "invoicing_cycle_configuration": {
                            "duration": 0,
                            "duration_unit": "day",
                        },
                        "metadata": {"foo": "string"},
                        "reference_id": "reference_id",
                    },
                }
            ],
            remove_adjustments=[
                {
                    "adjustment_id": "adjustment_id",
                    "plan_phase_order": 0,
                }
            ],
            remove_prices=[
                {
                    "price_id": "price_id",
                    "plan_phase_order": 0,
                }
            ],
            replace_adjustments=[
                {
                    "adjustment": {
                        "adjustment_type": "percentage_discount",
                        "percentage_discount": 0,
                        "applies_to_all": True,
                        "applies_to_item_ids": ["item_1", "item_2"],
                        "applies_to_price_ids": ["price_1", "price_2"],
                        "currency": "currency",
                        "filters": [
                            {
                                "field": "price_id",
                                "operator": "includes",
                                "values": ["string"],
                            }
                        ],
                        "is_invoice_level": True,
                        "price_type": "usage",
                    },
                    "replaces_adjustment_id": "replaces_adjustment_id",
                    "plan_phase_order": 0,
                }
            ],
            replace_prices=[
                {
                    "replaces_price_id": "replaces_price_id",
                    "allocation_price": {
                        "amount": "10.00",
                        "cadence": "monthly",
                        "currency": "USD",
                        "custom_expiration": {
                            "duration": 0,
                            "duration_unit": "day",
                        },
                        "expires_at_end_of_cadence": True,
                    },
                    "plan_phase_order": 0,
                    "price": {
                        "cadence": "annual",
                        "item_id": "item_id",
                        "model_type": "unit",
                        "name": "Annual fee",
                        "unit_config": {
                            "unit_amount": "unit_amount",
                            "scaling_factor": 0,
                        },
                        "billable_metric_id": "billable_metric_id",
                        "billed_in_advance": True,
                        "billing_cycle_configuration": {
                            "duration": 0,
                            "duration_unit": "day",
                        },
                        "conversion_rate": 0,
                        "conversion_rate_config": {
                            "conversion_rate_type": "unit",
                            "unit_config": {"unit_amount": "unit_amount"},
                        },
                        "currency": "currency",
                        "dimensional_price_configuration": {
                            "dimension_values": ["string"],
                            "dimensional_price_group_id": "dimensional_price_group_id",
                            "external_dimensional_price_group_id": "external_dimensional_price_group_id",
                        },
                        "external_price_id": "external_price_id",
                        "fixed_price_quantity": 0,
                        "invoice_grouping_key": "x",
                        "invoicing_cycle_configuration": {
                            "duration": 0,
                            "duration_unit": "day",
                        },
                        "metadata": {"foo": "string"},
                        "reference_id": "reference_id",
                    },
                }
            ],
            set_as_default=True,
        )
        assert_matches_type(PlanVersion, beta, path=["response"])

    @parametrize
    async def test_raw_response_create_plan_version(self, async_client: AsyncOrb) -> None:
        response = await async_client.beta.with_raw_response.create_plan_version(
            plan_id="plan_id",
            version=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        beta = response.parse()
        assert_matches_type(PlanVersion, beta, path=["response"])

    @parametrize
    async def test_streaming_response_create_plan_version(self, async_client: AsyncOrb) -> None:
        async with async_client.beta.with_streaming_response.create_plan_version(
            plan_id="plan_id",
            version=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            beta = await response.parse()
            assert_matches_type(PlanVersion, beta, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create_plan_version(self, async_client: AsyncOrb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `plan_id` but received ''"):
            await async_client.beta.with_raw_response.create_plan_version(
                plan_id="",
                version=0,
            )

    @parametrize
    async def test_method_fetch_plan_version(self, async_client: AsyncOrb) -> None:
        beta = await async_client.beta.fetch_plan_version(
            version="version",
            plan_id="plan_id",
        )
        assert_matches_type(PlanVersion, beta, path=["response"])

    @parametrize
    async def test_raw_response_fetch_plan_version(self, async_client: AsyncOrb) -> None:
        response = await async_client.beta.with_raw_response.fetch_plan_version(
            version="version",
            plan_id="plan_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        beta = response.parse()
        assert_matches_type(PlanVersion, beta, path=["response"])

    @parametrize
    async def test_streaming_response_fetch_plan_version(self, async_client: AsyncOrb) -> None:
        async with async_client.beta.with_streaming_response.fetch_plan_version(
            version="version",
            plan_id="plan_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            beta = await response.parse()
            assert_matches_type(PlanVersion, beta, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_fetch_plan_version(self, async_client: AsyncOrb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `plan_id` but received ''"):
            await async_client.beta.with_raw_response.fetch_plan_version(
                version="version",
                plan_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `version` but received ''"):
            await async_client.beta.with_raw_response.fetch_plan_version(
                version="",
                plan_id="plan_id",
            )

    @parametrize
    async def test_method_set_default_plan_version(self, async_client: AsyncOrb) -> None:
        beta = await async_client.beta.set_default_plan_version(
            plan_id="plan_id",
            version=0,
        )
        assert_matches_type(Plan, beta, path=["response"])

    @parametrize
    async def test_raw_response_set_default_plan_version(self, async_client: AsyncOrb) -> None:
        response = await async_client.beta.with_raw_response.set_default_plan_version(
            plan_id="plan_id",
            version=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        beta = response.parse()
        assert_matches_type(Plan, beta, path=["response"])

    @parametrize
    async def test_streaming_response_set_default_plan_version(self, async_client: AsyncOrb) -> None:
        async with async_client.beta.with_streaming_response.set_default_plan_version(
            plan_id="plan_id",
            version=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            beta = await response.parse()
            assert_matches_type(Plan, beta, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_set_default_plan_version(self, async_client: AsyncOrb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `plan_id` but received ''"):
            await async_client.beta.with_raw_response.set_default_plan_version(
                plan_id="",
                version=0,
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from .conversion_rate_tiered_config import ConversionRateTieredConfig

__all__ = ["TieredConversionRateConfig"]


class TieredConversionRateConfig(TypedDict, total=False):
    conversion_rate_type: Required[Literal["tiered"]]

    tiered_config: Required[ConversionRateTieredConfig]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from .custom_expiration import CustomExpiration

__all__ = ["Allocation"]


class Allocation(BaseModel):
    allows_rollover: bool

    currency: str

    custom_expiration: Optional[CustomExpiration] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from orb import Orb, AsyncOrb
from orb._utils import parse_datetime
from tests.utils import assert_matches_type
from orb.pagination import SyncPage, AsyncPage
from orb.types.events import (
    BackfillListResponse,
    BackfillCloseResponse,
    BackfillFetchResponse,
    BackfillCreateResponse,
    BackfillRevertResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestBackfills:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Orb) -> None:
        backfill = client.events.backfills.create(
            timeframe_end=parse_datetime("2019-12-27T18:11:19.117Z"),
            timeframe_start=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(BackfillCreateResponse, backfill, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Orb) -> None:
        backfill = client.events.backfills.create(
            timeframe_end=parse_datetime("2019-12-27T18:11:19.117Z"),
            timeframe_start=parse_datetime("2019-12-27T18:11:19.117Z"),
            close_time=parse_datetime("2019-12-27T18:11:19.117Z"),
            customer_id="customer_id",
            deprecation_filter="my_numeric_property > 100 AND my_other_property = 'bar'",
            external_customer_id="external_customer_id",
            replace_existing_events=True,
        )
        assert_matches_type(BackfillCreateResponse, backfill, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Orb) -> None:
        response = client.events.backfills.with_raw_response.create(
            timeframe_end=parse_datetime("2019-12-27T18:11:19.117Z"),
            timeframe_start=parse_datetime("2019-12-27T18:11:19.117Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        backfill = response.parse()
        assert_matches_type(BackfillCreateResponse, backfill, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Orb) -> None:
        with client.events.backfills.with_streaming_response.create(
            timeframe_end=parse_datetime("2019-12-27T18:11:19.117Z"),
            timeframe_start=parse_datetime("2019-12-27T18:11:19.117Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            backfill = response.parse()
            assert_matches_type(BackfillCreateResponse, backfill, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_list(self, client: Orb) -> None:
        backfill = client.events.backfills.list()
        assert_matches_type(SyncPage[BackfillListResponse], backfill, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Orb) -> None:
        backfill = client.events.backfills.list(
            cursor="cursor",
            limit=1,
        )
        assert_matches_type(SyncPage[BackfillListResponse], backfill, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Orb) -> None:
        response = client.events.backfills.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        backfill = response.parse()
        assert_matches_type(SyncPage[BackfillListResponse], backfill, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Orb) -> None:
        with client.events.backfills.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            backfill = response.parse()
            assert_matches_type(SyncPage[BackfillListResponse], backfill, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_close(self, client: Orb) -> None:
        backfill = client.events.backfills.close(
            "backfill_id",
        )
        assert_matches_type(BackfillCloseResponse, backfill, path=["response"])

    @parametrize
    def test_raw_response_close(self, client: Orb) -> None:
        response = client.events.backfills.with_raw_response.close(
            "backfill_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        backfill = response.parse()
        assert_matches_type(BackfillCloseResponse, backfill, path=["response"])

    @parametrize
    def test_streaming_response_close(self, client: Orb) -> None:
        with client.events.backfills.with_streaming_response.close(
            "backfill_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            backfill = response.parse()
            assert_matches_type(BackfillCloseResponse, backfill, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_close(self, client: Orb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `backfill_id` but received ''"):
            client.events.backfills.with_raw_response.close(
                "",
            )

    @parametrize
    def test_method_fetch(self, client: Orb) -> None:
        backfill = client.events.backfills.fetch(
            "backfill_id",
        )
        assert_matches_type(BackfillFetchResponse, backfill, path=["response"])

    @parametrize
    def test_raw_response_fetch(self, client: Orb) -> None:
        response = client.events.backfills.with_raw_response.fetch(
            "backfill_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        backfill = response.parse()
        assert_matches_type(BackfillFetchResponse, backfill, path=["response"])

    @parametrize
    def test_streaming_response_fetch(self, client: Orb) -> None:
        with client.events.backfills.with_streaming_response.fetch(
            "backfill_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            backfill = response.parse()
            assert_matches_type(BackfillFetchResponse, backfill, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_fetch(self, client: Orb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `backfill_id` but received ''"):
            client.events.backfills.with_raw_response.fetch(
                "",
            )

    @parametrize
    def test_method_revert(self, client: Orb) -> None:
        backfill = client.events.backfills.revert(
            "backfill_id",
        )
        assert_matches_type(BackfillRevertResponse, backfill, path=["response"])

    @parametrize
    def test_raw_response_revert(self, client: Orb) -> None:
        response = client.events.backfills.with_raw_response.revert(
            "backfill_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        backfill = response.parse()
        assert_matches_type(BackfillRevertResponse, backfill, path=["response"])

    @parametrize
    def test_streaming_response_revert(self, client: Orb) -> None:
        with client.events.backfills.with_streaming_response.revert(
            "backfill_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            backfill = response.parse()
            assert_matches_type(BackfillRevertResponse, backfill, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_revert(self, client: Orb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `backfill_id` but received ''"):
            client.events.backfills.with_raw_response.revert(
                "",
            )


class TestAsyncBackfills:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncOrb) -> None:
        backfill = await async_client.events.backfills.create(
            timeframe_end=parse_datetime("2019-12-27T18:11:19.117Z"),
            timeframe_start=parse_datetime("2019-12-27T18:11:19.117Z"),
        )
        assert_matches_type(BackfillCreateResponse, backfill, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncOrb) -> None:
        backfill = await async_client.events.backfills.create(
            timeframe_end=parse_datetime("2019-12-27T18:11:19.117Z"),
            timeframe_start=parse_datetime("2019-12-27T18:11:19.117Z"),
            close_time=parse_datetime("2019-12-27T18:11:19.117Z"),
            customer_id="customer_id",
            deprecation_filter="my_numeric_property > 100 AND my_other_property = 'bar'",
            external_customer_id="external_customer_id",
            replace_existing_events=True,
        )
        assert_matches_type(BackfillCreateResponse, backfill, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncOrb) -> None:
        response = await async_client.events.backfills.with_raw_response.create(
            timeframe_end=parse_datetime("2019-12-27T18:11:19.117Z"),
            timeframe_start=parse_datetime("2019-12-27T18:11:19.117Z"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        backfill = response.parse()
        assert_matches_type(BackfillCreateResponse, backfill, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncOrb) -> None:
        async with async_client.events.backfills.with_streaming_response.create(
            timeframe_end=parse_datetime("2019-12-27T18:11:19.117Z"),
            timeframe_start=parse_datetime("2019-12-27T18:11:19.117Z"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            backfill = await response.parse()
            assert_matches_type(BackfillCreateResponse, backfill, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_list(self, async_client: AsyncOrb) -> None:
        backfill = await async_client.events.backfills.list()
        assert_matches_type(AsyncPage[BackfillListResponse], backfill, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncOrb) -> None:
        backfill = await async_client.events.backfills.list(
            cursor="cursor",
            limit=1,
        )
        assert_matches_type(AsyncPage[BackfillListResponse], backfill, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncOrb) -> None:
        response = await async_client.events.backfills.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        backfill = response.parse()
        assert_matches_type(AsyncPage[BackfillListResponse], backfill, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncOrb) -> None:
        async with async_client.events.backfills.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            backfill = await response.parse()
            assert_matches_type(AsyncPage[BackfillListResponse], backfill, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_close(self, async_client: AsyncOrb) -> None:
        backfill = await async_client.events.backfills.close(
            "backfill_id",
        )
        assert_matches_type(BackfillCloseResponse, backfill, path=["response"])

    @parametrize
    async def test_raw_response_close(self, async_client: AsyncOrb) -> None:
        response = await async_client.events.backfills.with_raw_response.close(
            "backfill_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        backfill = response.parse()
        assert_matches_type(BackfillCloseResponse, backfill, path=["response"])

    @parametrize
    async def test_streaming_response_close(self, async_client: AsyncOrb) -> None:
        async with async_client.events.backfills.with_streaming_response.close(
            "backfill_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            backfill = await response.parse()
            assert_matches_type(BackfillCloseResponse, backfill, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_close(self, async_client: AsyncOrb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `backfill_id` but received ''"):
            await async_client.events.backfills.with_raw_response.close(
                "",
            )

    @parametrize
    async def test_method_fetch(self, async_client: AsyncOrb) -> None:
        backfill = await async_client.events.backfills.fetch(
            "backfill_id",
        )
        assert_matches_type(BackfillFetchResponse, backfill, path=["response"])

    @parametrize
    async def test_raw_response_fetch(self, async_client: AsyncOrb) -> None:
        response = await async_client.events.backfills.with_raw_response.fetch(
            "backfill_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        backfill = response.parse()
        assert_matches_type(BackfillFetchResponse, backfill, path=["response"])

    @parametrize
    async def test_streaming_response_fetch(self, async_client: AsyncOrb) -> None:
        async with async_client.events.backfills.with_streaming_response.fetch(
            "backfill_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            backfill = await response.parse()
            assert_matches_type(BackfillFetchResponse, backfill, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_fetch(self, async_client: AsyncOrb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `backfill_id` but received ''"):
            await async_client.events.backfills.with_raw_response.fetch(
                "",
            )

    @parametrize
    async def test_method_revert(self, async_client: AsyncOrb) -> None:
        backfill = await async_client.events.backfills.revert(
            "backfill_id",
        )
        assert_matches_type(BackfillRevertResponse, backfill, path=["response"])

    @parametrize
    async def test_raw_response_revert(self, async_client: AsyncOrb) -> None:
        response = await async_client.events.backfills.with_raw_response.revert(
            "backfill_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        backfill = response.parse()
        assert_matches_type(BackfillRevertResponse, backfill, path=["response"])

    @parametrize
    async def test_streaming_response_revert(self, async_client: AsyncOrb) -> None:
        async with async_client.events.backfills.with_streaming_response.revert(
            "backfill_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            backfill = await response.parse()
            assert_matches_type(BackfillRevertResponse, backfill, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_revert(self, async_client: AsyncOrb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `backfill_id` but received ''"):
            await async_client.events.backfills.with_raw_response.revert(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["SubLineItemMatrixConfig"]


class SubLineItemMatrixConfig(BaseModel):
    dimension_values: List[Optional[str]]
    """The ordered dimension values for this line item."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

from .._types import SequenceNotStr

__all__ = ["CustomerHierarchyConfigParam"]


class CustomerHierarchyConfigParam(TypedDict, total=False):
    child_customer_ids: SequenceNotStr[str]
    """A list of child customer IDs to add to the hierarchy.

    The desired child customers must not already be part of another hierarchy.
    """

    parent_customer_id: Optional[str]
    """The ID of the parent customer in the hierarchy.

    The desired parent customer must not be a child of another customer.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Annotated, TypeAlias

from .._utils import PropertyInfo
from .._models import BaseModel
from .shared.price import Price
from .plan_version_phase import PlanVersionPhase
from .shared.plan_phase_maximum_adjustment import PlanPhaseMaximumAdjustment
from .shared.plan_phase_minimum_adjustment import PlanPhaseMinimumAdjustment
from .shared.plan_phase_usage_discount_adjustment import PlanPhaseUsageDiscountAdjustment
from .shared.plan_phase_amount_discount_adjustment import PlanPhaseAmountDiscountAdjustment
from .shared.plan_phase_percentage_discount_adjustment import PlanPhasePercentageDiscountAdjustment

__all__ = ["PlanVersion", "Adjustment"]

Adjustment: TypeAlias = Annotated[
    Union[
        PlanPhaseUsageDiscountAdjustment,
        PlanPhaseAmountDiscountAdjustment,
        PlanPhasePercentageDiscountAdjustment,
        PlanPhaseMinimumAdjustment,
        PlanPhaseMaximumAdjustment,
    ],
    PropertyInfo(discriminator="adjustment_type"),
]


class PlanVersion(BaseModel):
    adjustments: List[Adjustment]
    """Adjustments for this plan.

    If the plan has phases, this includes adjustments across all phases of the plan.
    """

    created_at: datetime

    plan_phases: Optional[List[PlanVersionPhase]] = None

    prices: List[Price]
    """Prices for this plan.

    If the plan has phases, this includes prices across all phases of the plan.
    """

    version: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["PackageConfig"]


class PackageConfig(TypedDict, total=False):
    package_amount: Required[str]
    """A currency amount to rate usage by"""

    package_size: Required[int]
    """An integer amount to represent package size.

    For example, 1000 here would divide usage by 1000 before multiplying by
    package_amount in rating
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from orb import Orb, AsyncOrb
from orb.types import (
    Alert,
)
from orb._utils import parse_datetime
from tests.utils import assert_matches_type
from orb.pagination import SyncPage, AsyncPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAlerts:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_retrieve(self, client: Orb) -> None:
        alert = client.alerts.retrieve(
            "alert_id",
        )
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Orb) -> None:
        response = client.alerts.with_raw_response.retrieve(
            "alert_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        alert = response.parse()
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Orb) -> None:
        with client.alerts.with_streaming_response.retrieve(
            "alert_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            alert = response.parse()
            assert_matches_type(Alert, alert, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Orb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `alert_id` but received ''"):
            client.alerts.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_update(self, client: Orb) -> None:
        alert = client.alerts.update(
            alert_configuration_id="alert_configuration_id",
            thresholds=[{"value": 0}],
        )
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: Orb) -> None:
        response = client.alerts.with_raw_response.update(
            alert_configuration_id="alert_configuration_id",
            thresholds=[{"value": 0}],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        alert = response.parse()
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: Orb) -> None:
        with client.alerts.with_streaming_response.update(
            alert_configuration_id="alert_configuration_id",
            thresholds=[{"value": 0}],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            alert = response.parse()
            assert_matches_type(Alert, alert, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: Orb) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `alert_configuration_id` but received ''"
        ):
            client.alerts.with_raw_response.update(
                alert_configuration_id="",
                thresholds=[{"value": 0}],
            )

    @pytest.mark.skip(reason="plan_version=0 breaks Prism")
    @parametrize
    def test_method_list(self, client: Orb) -> None:
        alert = client.alerts.list()
        assert_matches_type(SyncPage[Alert], alert, path=["response"])

    @pytest.mark.skip(reason="plan_version=0 breaks Prism")
    @parametrize
    def test_method_list_with_all_params(self, client: Orb) -> None:
        alert = client.alerts.list(
            created_at_gt=parse_datetime("2019-12-27T18:11:19.117Z"),
            created_at_gte=parse_datetime("2019-12-27T18:11:19.117Z"),
            created_at_lt=parse_datetime("2019-12-27T18:11:19.117Z"),
            created_at_lte=parse_datetime("2019-12-27T18:11:19.117Z"),
            cursor="cursor",
            customer_id="customer_id",
            external_customer_id="external_customer_id",
            limit=1,
            subscription_id="subscription_id",
        )
        assert_matches_type(SyncPage[Alert], alert, path=["response"])

    @pytest.mark.skip(reason="plan_version=0 breaks Prism")
    @parametrize
    def test_raw_response_list(self, client: Orb) -> None:
        response = client.alerts.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        alert = response.parse()
        assert_matches_type(SyncPage[Alert], alert, path=["response"])

    @pytest.mark.skip(reason="plan_version=0 breaks Prism")
    @parametrize
    def test_streaming_response_list(self, client: Orb) -> None:
        with client.alerts.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            alert = response.parse()
            assert_matches_type(SyncPage[Alert], alert, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_create_for_customer(self, client: Orb) -> None:
        alert = client.alerts.create_for_customer(
            customer_id="customer_id",
            currency="currency",
            type="credit_balance_depleted",
        )
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    def test_method_create_for_customer_with_all_params(self, client: Orb) -> None:
        alert = client.alerts.create_for_customer(
            customer_id="customer_id",
            currency="currency",
            type="credit_balance_depleted",
            thresholds=[{"value": 0}],
        )
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    def test_raw_response_create_for_customer(self, client: Orb) -> None:
        response = client.alerts.with_raw_response.create_for_customer(
            customer_id="customer_id",
            currency="currency",
            type="credit_balance_depleted",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        alert = response.parse()
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    def test_streaming_response_create_for_customer(self, client: Orb) -> None:
        with client.alerts.with_streaming_response.create_for_customer(
            customer_id="customer_id",
            currency="currency",
            type="credit_balance_depleted",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            alert = response.parse()
            assert_matches_type(Alert, alert, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create_for_customer(self, client: Orb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            client.alerts.with_raw_response.create_for_customer(
                customer_id="",
                currency="currency",
                type="credit_balance_depleted",
            )

    @parametrize
    def test_method_create_for_external_customer(self, client: Orb) -> None:
        alert = client.alerts.create_for_external_customer(
            external_customer_id="external_customer_id",
            currency="currency",
            type="credit_balance_depleted",
        )
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    def test_method_create_for_external_customer_with_all_params(self, client: Orb) -> None:
        alert = client.alerts.create_for_external_customer(
            external_customer_id="external_customer_id",
            currency="currency",
            type="credit_balance_depleted",
            thresholds=[{"value": 0}],
        )
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    def test_raw_response_create_for_external_customer(self, client: Orb) -> None:
        response = client.alerts.with_raw_response.create_for_external_customer(
            external_customer_id="external_customer_id",
            currency="currency",
            type="credit_balance_depleted",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        alert = response.parse()
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    def test_streaming_response_create_for_external_customer(self, client: Orb) -> None:
        with client.alerts.with_streaming_response.create_for_external_customer(
            external_customer_id="external_customer_id",
            currency="currency",
            type="credit_balance_depleted",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            alert = response.parse()
            assert_matches_type(Alert, alert, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create_for_external_customer(self, client: Orb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `external_customer_id` but received ''"):
            client.alerts.with_raw_response.create_for_external_customer(
                external_customer_id="",
                currency="currency",
                type="credit_balance_depleted",
            )

    @parametrize
    def test_method_create_for_subscription(self, client: Orb) -> None:
        alert = client.alerts.create_for_subscription(
            subscription_id="subscription_id",
            thresholds=[{"value": 0}],
            type="usage_exceeded",
        )
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    def test_method_create_for_subscription_with_all_params(self, client: Orb) -> None:
        alert = client.alerts.create_for_subscription(
            subscription_id="subscription_id",
            thresholds=[{"value": 0}],
            type="usage_exceeded",
            metric_id="metric_id",
        )
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    def test_raw_response_create_for_subscription(self, client: Orb) -> None:
        response = client.alerts.with_raw_response.create_for_subscription(
            subscription_id="subscription_id",
            thresholds=[{"value": 0}],
            type="usage_exceeded",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        alert = response.parse()
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    def test_streaming_response_create_for_subscription(self, client: Orb) -> None:
        with client.alerts.with_streaming_response.create_for_subscription(
            subscription_id="subscription_id",
            thresholds=[{"value": 0}],
            type="usage_exceeded",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            alert = response.parse()
            assert_matches_type(Alert, alert, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_create_for_subscription(self, client: Orb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `subscription_id` but received ''"):
            client.alerts.with_raw_response.create_for_subscription(
                subscription_id="",
                thresholds=[{"value": 0}],
                type="usage_exceeded",
            )

    @parametrize
    def test_method_disable(self, client: Orb) -> None:
        alert = client.alerts.disable(
            alert_configuration_id="alert_configuration_id",
        )
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    def test_method_disable_with_all_params(self, client: Orb) -> None:
        alert = client.alerts.disable(
            alert_configuration_id="alert_configuration_id",
            subscription_id="subscription_id",
        )
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    def test_raw_response_disable(self, client: Orb) -> None:
        response = client.alerts.with_raw_response.disable(
            alert_configuration_id="alert_configuration_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        alert = response.parse()
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    def test_streaming_response_disable(self, client: Orb) -> None:
        with client.alerts.with_streaming_response.disable(
            alert_configuration_id="alert_configuration_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            alert = response.parse()
            assert_matches_type(Alert, alert, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_disable(self, client: Orb) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `alert_configuration_id` but received ''"
        ):
            client.alerts.with_raw_response.disable(
                alert_configuration_id="",
            )

    @parametrize
    def test_method_enable(self, client: Orb) -> None:
        alert = client.alerts.enable(
            alert_configuration_id="alert_configuration_id",
        )
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    def test_method_enable_with_all_params(self, client: Orb) -> None:
        alert = client.alerts.enable(
            alert_configuration_id="alert_configuration_id",
            subscription_id="subscription_id",
        )
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    def test_raw_response_enable(self, client: Orb) -> None:
        response = client.alerts.with_raw_response.enable(
            alert_configuration_id="alert_configuration_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        alert = response.parse()
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    def test_streaming_response_enable(self, client: Orb) -> None:
        with client.alerts.with_streaming_response.enable(
            alert_configuration_id="alert_configuration_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            alert = response.parse()
            assert_matches_type(Alert, alert, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_enable(self, client: Orb) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `alert_configuration_id` but received ''"
        ):
            client.alerts.with_raw_response.enable(
                alert_configuration_id="",
            )


class TestAsyncAlerts:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncOrb) -> None:
        alert = await async_client.alerts.retrieve(
            "alert_id",
        )
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncOrb) -> None:
        response = await async_client.alerts.with_raw_response.retrieve(
            "alert_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        alert = response.parse()
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncOrb) -> None:
        async with async_client.alerts.with_streaming_response.retrieve(
            "alert_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            alert = await response.parse()
            assert_matches_type(Alert, alert, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncOrb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `alert_id` but received ''"):
            await async_client.alerts.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncOrb) -> None:
        alert = await async_client.alerts.update(
            alert_configuration_id="alert_configuration_id",
            thresholds=[{"value": 0}],
        )
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncOrb) -> None:
        response = await async_client.alerts.with_raw_response.update(
            alert_configuration_id="alert_configuration_id",
            thresholds=[{"value": 0}],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        alert = response.parse()
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncOrb) -> None:
        async with async_client.alerts.with_streaming_response.update(
            alert_configuration_id="alert_configuration_id",
            thresholds=[{"value": 0}],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            alert = await response.parse()
            assert_matches_type(Alert, alert, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncOrb) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `alert_configuration_id` but received ''"
        ):
            await async_client.alerts.with_raw_response.update(
                alert_configuration_id="",
                thresholds=[{"value": 0}],
            )

    @pytest.mark.skip(reason="plan_version=0 breaks Prism")
    @parametrize
    async def test_method_list(self, async_client: AsyncOrb) -> None:
        alert = await async_client.alerts.list()
        assert_matches_type(AsyncPage[Alert], alert, path=["response"])

    @pytest.mark.skip(reason="plan_version=0 breaks Prism")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncOrb) -> None:
        alert = await async_client.alerts.list(
            created_at_gt=parse_datetime("2019-12-27T18:11:19.117Z"),
            created_at_gte=parse_datetime("2019-12-27T18:11:19.117Z"),
            created_at_lt=parse_datetime("2019-12-27T18:11:19.117Z"),
            created_at_lte=parse_datetime("2019-12-27T18:11:19.117Z"),
            cursor="cursor",
            customer_id="customer_id",
            external_customer_id="external_customer_id",
            limit=1,
            subscription_id="subscription_id",
        )
        assert_matches_type(AsyncPage[Alert], alert, path=["response"])

    @pytest.mark.skip(reason="plan_version=0 breaks Prism")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncOrb) -> None:
        response = await async_client.alerts.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        alert = response.parse()
        assert_matches_type(AsyncPage[Alert], alert, path=["response"])

    @pytest.mark.skip(reason="plan_version=0 breaks Prism")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncOrb) -> None:
        async with async_client.alerts.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            alert = await response.parse()
            assert_matches_type(AsyncPage[Alert], alert, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_create_for_customer(self, async_client: AsyncOrb) -> None:
        alert = await async_client.alerts.create_for_customer(
            customer_id="customer_id",
            currency="currency",
            type="credit_balance_depleted",
        )
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    async def test_method_create_for_customer_with_all_params(self, async_client: AsyncOrb) -> None:
        alert = await async_client.alerts.create_for_customer(
            customer_id="customer_id",
            currency="currency",
            type="credit_balance_depleted",
            thresholds=[{"value": 0}],
        )
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    async def test_raw_response_create_for_customer(self, async_client: AsyncOrb) -> None:
        response = await async_client.alerts.with_raw_response.create_for_customer(
            customer_id="customer_id",
            currency="currency",
            type="credit_balance_depleted",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        alert = response.parse()
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    async def test_streaming_response_create_for_customer(self, async_client: AsyncOrb) -> None:
        async with async_client.alerts.with_streaming_response.create_for_customer(
            customer_id="customer_id",
            currency="currency",
            type="credit_balance_depleted",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            alert = await response.parse()
            assert_matches_type(Alert, alert, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create_for_customer(self, async_client: AsyncOrb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `customer_id` but received ''"):
            await async_client.alerts.with_raw_response.create_for_customer(
                customer_id="",
                currency="currency",
                type="credit_balance_depleted",
            )

    @parametrize
    async def test_method_create_for_external_customer(self, async_client: AsyncOrb) -> None:
        alert = await async_client.alerts.create_for_external_customer(
            external_customer_id="external_customer_id",
            currency="currency",
            type="credit_balance_depleted",
        )
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    async def test_method_create_for_external_customer_with_all_params(self, async_client: AsyncOrb) -> None:
        alert = await async_client.alerts.create_for_external_customer(
            external_customer_id="external_customer_id",
            currency="currency",
            type="credit_balance_depleted",
            thresholds=[{"value": 0}],
        )
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    async def test_raw_response_create_for_external_customer(self, async_client: AsyncOrb) -> None:
        response = await async_client.alerts.with_raw_response.create_for_external_customer(
            external_customer_id="external_customer_id",
            currency="currency",
            type="credit_balance_depleted",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        alert = response.parse()
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    async def test_streaming_response_create_for_external_customer(self, async_client: AsyncOrb) -> None:
        async with async_client.alerts.with_streaming_response.create_for_external_customer(
            external_customer_id="external_customer_id",
            currency="currency",
            type="credit_balance_depleted",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            alert = await response.parse()
            assert_matches_type(Alert, alert, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create_for_external_customer(self, async_client: AsyncOrb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `external_customer_id` but received ''"):
            await async_client.alerts.with_raw_response.create_for_external_customer(
                external_customer_id="",
                currency="currency",
                type="credit_balance_depleted",
            )

    @parametrize
    async def test_method_create_for_subscription(self, async_client: AsyncOrb) -> None:
        alert = await async_client.alerts.create_for_subscription(
            subscription_id="subscription_id",
            thresholds=[{"value": 0}],
            type="usage_exceeded",
        )
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    async def test_method_create_for_subscription_with_all_params(self, async_client: AsyncOrb) -> None:
        alert = await async_client.alerts.create_for_subscription(
            subscription_id="subscription_id",
            thresholds=[{"value": 0}],
            type="usage_exceeded",
            metric_id="metric_id",
        )
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    async def test_raw_response_create_for_subscription(self, async_client: AsyncOrb) -> None:
        response = await async_client.alerts.with_raw_response.create_for_subscription(
            subscription_id="subscription_id",
            thresholds=[{"value": 0}],
            type="usage_exceeded",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        alert = response.parse()
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    async def test_streaming_response_create_for_subscription(self, async_client: AsyncOrb) -> None:
        async with async_client.alerts.with_streaming_response.create_for_subscription(
            subscription_id="subscription_id",
            thresholds=[{"value": 0}],
            type="usage_exceeded",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            alert = await response.parse()
            assert_matches_type(Alert, alert, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_create_for_subscription(self, async_client: AsyncOrb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `subscription_id` but received ''"):
            await async_client.alerts.with_raw_response.create_for_subscription(
                subscription_id="",
                thresholds=[{"value": 0}],
                type="usage_exceeded",
            )

    @parametrize
    async def test_method_disable(self, async_client: AsyncOrb) -> None:
        alert = await async_client.alerts.disable(
            alert_configuration_id="alert_configuration_id",
        )
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    async def test_method_disable_with_all_params(self, async_client: AsyncOrb) -> None:
        alert = await async_client.alerts.disable(
            alert_configuration_id="alert_configuration_id",
            subscription_id="subscription_id",
        )
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    async def test_raw_response_disable(self, async_client: AsyncOrb) -> None:
        response = await async_client.alerts.with_raw_response.disable(
            alert_configuration_id="alert_configuration_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        alert = response.parse()
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    async def test_streaming_response_disable(self, async_client: AsyncOrb) -> None:
        async with async_client.alerts.with_streaming_response.disable(
            alert_configuration_id="alert_configuration_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            alert = await response.parse()
            assert_matches_type(Alert, alert, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_disable(self, async_client: AsyncOrb) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `alert_configuration_id` but received ''"
        ):
            await async_client.alerts.with_raw_response.disable(
                alert_configuration_id="",
            )

    @parametrize
    async def test_method_enable(self, async_client: AsyncOrb) -> None:
        alert = await async_client.alerts.enable(
            alert_configuration_id="alert_configuration_id",
        )
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    async def test_method_enable_with_all_params(self, async_client: AsyncOrb) -> None:
        alert = await async_client.alerts.enable(
            alert_configuration_id="alert_configuration_id",
            subscription_id="subscription_id",
        )
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    async def test_raw_response_enable(self, async_client: AsyncOrb) -> None:
        response = await async_client.alerts.with_raw_response.enable(
            alert_configuration_id="alert_configuration_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        alert = response.parse()
        assert_matches_type(Alert, alert, path=["response"])

    @parametrize
    async def test_streaming_response_enable(self, async_client: AsyncOrb) -> None:
        async with async_client.alerts.with_streaming_response.enable(
            alert_configuration_id="alert_configuration_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            alert = await response.parse()
            assert_matches_type(Alert, alert, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_enable(self, async_client: AsyncOrb) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `alert_configuration_id` but received ''"
        ):
            await async_client.alerts.with_raw_response.enable(
                alert_configuration_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from orb import Orb, AsyncOrb
from orb.types import Coupon
from tests.utils import assert_matches_type
from orb.pagination import SyncPage, AsyncPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCoupons:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Orb) -> None:
        coupon = client.coupons.create(
            discount={
                "discount_type": "percentage",
                "percentage_discount": 0,
            },
            redemption_code="HALFOFF",
        )
        assert_matches_type(Coupon, coupon, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Orb) -> None:
        coupon = client.coupons.create(
            discount={
                "discount_type": "percentage",
                "percentage_discount": 0,
            },
            redemption_code="HALFOFF",
            duration_in_months=12,
            max_redemptions=1,
        )
        assert_matches_type(Coupon, coupon, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Orb) -> None:
        response = client.coupons.with_raw_response.create(
            discount={
                "discount_type": "percentage",
                "percentage_discount": 0,
            },
            redemption_code="HALFOFF",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        coupon = response.parse()
        assert_matches_type(Coupon, coupon, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Orb) -> None:
        with client.coupons.with_streaming_response.create(
            discount={
                "discount_type": "percentage",
                "percentage_discount": 0,
            },
            redemption_code="HALFOFF",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            coupon = response.parse()
            assert_matches_type(Coupon, coupon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_list(self, client: Orb) -> None:
        coupon = client.coupons.list()
        assert_matches_type(SyncPage[Coupon], coupon, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Orb) -> None:
        coupon = client.coupons.list(
            cursor="cursor",
            limit=1,
            redemption_code="redemption_code",
            show_archived=True,
        )
        assert_matches_type(SyncPage[Coupon], coupon, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Orb) -> None:
        response = client.coupons.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        coupon = response.parse()
        assert_matches_type(SyncPage[Coupon], coupon, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Orb) -> None:
        with client.coupons.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            coupon = response.parse()
            assert_matches_type(SyncPage[Coupon], coupon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_archive(self, client: Orb) -> None:
        coupon = client.coupons.archive(
            "coupon_id",
        )
        assert_matches_type(Coupon, coupon, path=["response"])

    @parametrize
    def test_raw_response_archive(self, client: Orb) -> None:
        response = client.coupons.with_raw_response.archive(
            "coupon_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        coupon = response.parse()
        assert_matches_type(Coupon, coupon, path=["response"])

    @parametrize
    def test_streaming_response_archive(self, client: Orb) -> None:
        with client.coupons.with_streaming_response.archive(
            "coupon_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            coupon = response.parse()
            assert_matches_type(Coupon, coupon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_archive(self, client: Orb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `coupon_id` but received ''"):
            client.coupons.with_raw_response.archive(
                "",
            )

    @parametrize
    def test_method_fetch(self, client: Orb) -> None:
        coupon = client.coupons.fetch(
            "coupon_id",
        )
        assert_matches_type(Coupon, coupon, path=["response"])

    @parametrize
    def test_raw_response_fetch(self, client: Orb) -> None:
        response = client.coupons.with_raw_response.fetch(
            "coupon_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        coupon = response.parse()
        assert_matches_type(Coupon, coupon, path=["response"])

    @parametrize
    def test_streaming_response_fetch(self, client: Orb) -> None:
        with client.coupons.with_streaming_response.fetch(
            "coupon_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            coupon = response.parse()
            assert_matches_type(Coupon, coupon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_fetch(self, client: Orb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `coupon_id` but received ''"):
            client.coupons.with_raw_response.fetch(
                "",
            )


class TestAsyncCoupons:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncOrb) -> None:
        coupon = await async_client.coupons.create(
            discount={
                "discount_type": "percentage",
                "percentage_discount": 0,
            },
            redemption_code="HALFOFF",
        )
        assert_matches_type(Coupon, coupon, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncOrb) -> None:
        coupon = await async_client.coupons.create(
            discount={
                "discount_type": "percentage",
                "percentage_discount": 0,
            },
            redemption_code="HALFOFF",
            duration_in_months=12,
            max_redemptions=1,
        )
        assert_matches_type(Coupon, coupon, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncOrb) -> None:
        response = await async_client.coupons.with_raw_response.create(
            discount={
                "discount_type": "percentage",
                "percentage_discount": 0,
            },
            redemption_code="HALFOFF",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        coupon = response.parse()
        assert_matches_type(Coupon, coupon, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncOrb) -> None:
        async with async_client.coupons.with_streaming_response.create(
            discount={
                "discount_type": "percentage",
                "percentage_discount": 0,
            },
            redemption_code="HALFOFF",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            coupon = await response.parse()
            assert_matches_type(Coupon, coupon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_list(self, async_client: AsyncOrb) -> None:
        coupon = await async_client.coupons.list()
        assert_matches_type(AsyncPage[Coupon], coupon, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncOrb) -> None:
        coupon = await async_client.coupons.list(
            cursor="cursor",
            limit=1,
            redemption_code="redemption_code",
            show_archived=True,
        )
        assert_matches_type(AsyncPage[Coupon], coupon, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncOrb) -> None:
        response = await async_client.coupons.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        coupon = response.parse()
        assert_matches_type(AsyncPage[Coupon], coupon, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncOrb) -> None:
        async with async_client.coupons.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            coupon = await response.parse()
            assert_matches_type(AsyncPage[Coupon], coupon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_archive(self, async_client: AsyncOrb) -> None:
        coupon = await async_client.coupons.archive(
            "coupon_id",
        )
        assert_matches_type(Coupon, coupon, path=["response"])

    @parametrize
    async def test_raw_response_archive(self, async_client: AsyncOrb) -> None:
        response = await async_client.coupons.with_raw_response.archive(
            "coupon_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        coupon = response.parse()
        assert_matches_type(Coupon, coupon, path=["response"])

    @parametrize
    async def test_streaming_response_archive(self, async_client: AsyncOrb) -> None:
        async with async_client.coupons.with_streaming_response.archive(
            "coupon_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            coupon = await response.parse()
            assert_matches_type(Coupon, coupon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_archive(self, async_client: AsyncOrb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `coupon_id` but received ''"):
            await async_client.coupons.with_raw_response.archive(
                "",
            )

    @parametrize
    async def test_method_fetch(self, async_client: AsyncOrb) -> None:
        coupon = await async_client.coupons.fetch(
            "coupon_id",
        )
        assert_matches_type(Coupon, coupon, path=["response"])

    @parametrize
    async def test_raw_response_fetch(self, async_client: AsyncOrb) -> None:
        response = await async_client.coupons.with_raw_response.fetch(
            "coupon_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        coupon = response.parse()
        assert_matches_type(Coupon, coupon, path=["response"])

    @parametrize
    async def test_streaming_response_fetch(self, async_client: AsyncOrb) -> None:
        async with async_client.coupons.with_streaming_response.fetch(
            "coupon_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            coupon = await response.parse()
            assert_matches_type(Coupon, coupon, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_fetch(self, async_client: AsyncOrb) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `coupon_id` but received ''"):
            await async_client.coupons.with_raw_response.fetch(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from orb import Orb, AsyncOrb
from orb.types import InvoiceLineItemCreateResponse
from orb._utils import parse_date
from tests.utils import assert_matches_type

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestInvoiceLineItems:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Orb) -> None:
        invoice_line_item = client.invoice_line_items.create(
            amount="12.00",
            end_date=parse_date("2023-09-22"),
            invoice_id="4khy3nwzktxv7",
            name="Item Name",
            quantity=1,
            start_date=parse_date("2023-09-22"),
        )
        assert_matches_type(InvoiceLineItemCreateResponse, invoice_line_item, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Orb) -> None:
        response = client.invoice_line_items.with_raw_response.create(
            amount="12.00",
            end_date=parse_date("2023-09-22"),
            invoice_id="4khy3nwzktxv7",
            name="Item Name",
            quantity=1,
            start_date=parse_date("2023-09-22"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        invoice_line_item = response.parse()
        assert_matches_type(InvoiceLineItemCreateResponse, invoice_line_item, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Orb) -> None:
        with client.invoice_line_items.with_streaming_response.create(
            amount="12.00",
            end_date=parse_date("2023-09-22"),
            invoice_id="4khy3nwzktxv7",
            name="Item Name",
            quantity=1,
            start_date=parse_date("2023-09-22"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            invoice_line_item = response.parse()
            assert_matches_type(InvoiceLineItemCreateResponse, invoice_line_item, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncInvoiceLineItems:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncOrb) -> None:
        invoice_line_item = await async_client.invoice_line_items.create(
            amount="12.00",
            end_date=parse_date("2023-09-22"),
            invoice_id="4khy3nwzktxv7",
            name="Item Name",
            quantity=1,
            start_date=parse_date("2023-09-22"),
        )
        assert_matches_type(InvoiceLineItemCreateResponse, invoice_line_item, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncOrb) -> None:
        response = await async_client.invoice_line_items.with_raw_response.create(
            amount="12.00",
            end_date=parse_date("2023-09-22"),
            invoice_id="4khy3nwzktxv7",
            name="Item Name",
            quantity=1,
            start_date=parse_date("2023-09-22"),
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        invoice_line_item = response.parse()
        assert_matches_type(InvoiceLineItemCreateResponse, invoice_line_item, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncOrb) -> None:
        async with async_client.invoice_line_items.with_streaming_response.create(
            amount="12.00",
            end_date=parse_date("2023-09-22"),
            invoice_id="4khy3nwzktxv7",
            name="Item Name",
            quantity=1,
            start_date=parse_date("2023-09-22"),
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            invoice_line_item = await response.parse()
            assert_matches_type(InvoiceLineItemCreateResponse, invoice_line_item, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ...._models import BaseModel

__all__ = ["AffectedBlock"]


class AffectedBlock(BaseModel):
    id: str

    expiry_date: Optional[datetime] = None

    per_unit_cost_basis: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["TransformPriceFilter"]


class TransformPriceFilter(TypedDict, total=False):
    field: Required[Literal["price_id", "item_id", "price_type", "currency", "pricing_unit_id"]]
    """The property of the price to filter on."""

    operator: Required[Literal["includes", "excludes"]]
    """Should prices that match the filter be included or excluded."""

    values: Required[SequenceNotStr[str]]
    """The IDs or values that match this filter."""
